/* /////////////////////////////////////////////////////////////////////////////
 * File:    mmap_ex.c
 *
 * Purpose: Implementation of the mmap_file function; platform-independent.
 *
 * Created  18th December 2003
 * Updated: 19th September 2005
 *
 * Home:    http://synesis.com.au/software/
 *
 * Copyright (c) 2002-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the names of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#define _SYNSOFT_VER_C_MMAP_EX_MAJOR    2
#define _SYNSOFT_VER_C_MMAP_EX_MINOR    0
#define _SYNSOFT_VER_C_MMAP_EX_REVISION	1
#define _SYNSOFT_VER_C_MMAP_EX_EDIT     7

#include <stddef.h>

#include <fcntl.h>
#include <io.h>
#if !defined(SEEK_END) || \
    !defined(SEEK_SET)
# include <stdio.h>
#endif /* !SEEK_END || !SEEK_SET */
#include <share.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/stat.h>

#if defined(__BORLANDC__) || \
    defined(__DMC__)
#elif defined(__INTEL_COMPILER) || \
	  defined(__MWERKS__) || \
      defined(_MSC_VER)
# define lseek	_lseek
# define open	_open
# define write	_write
#else
# error Compiler not discriminated
#endif /* compiler */


void *mmap_file(char const  *fileName   /* = NULL */
            ,   size_t      len
			,	size_t		offset
            ,   int         prot
            ,   int         flags
			,	size_t		*plen)
{
    void    *pvMap;
    int     fd;
    int     oflag;
    int     pmode;

    flags = 0; /* Ignored for the current implentation */

    if( NULL == fileName ||
        PROT_WRITE == (prot & PROT_WRITE))
    {
        prot    =   PROT_READ | PROT_WRITE;
        oflag   =   O_CREAT | O_BINARY | O_RDWR;
        pmode   =   SH_DENYRW | S_IREAD | S_IWRITE;
    }
    else
    {
        prot    =   PROT_READ;
        oflag   =   O_BINARY | O_RDONLY;
        pmode   =   SH_DENYWR | S_IREAD;
    }

    if(NULL == fileName)
    {
        flags |= MAP_ANONYMOUS;

        if(0 == len)
        {
            return MAP_FAILED;
        }
        else
        {
            size_t  pagesize    =   getpagesize();

            len = (len + pagesize - 1) & ~(pagesize - 1);
        }

        fd = -1;
    }
    else
    {
        size_t  fileLen;

#if defined(__BORLANDC__)
        fd = open(fileName, oflag);

        ((void)pmode);
#else
        fd = open(fileName, oflag, pmode);
#endif /* __SYNSOFT_DBS_DEBUG */

        if(fd < 0)
        {
            return MAP_FAILED;
        }

        fileLen = lseek(fd, 0, SEEK_END);
        lseek(fd, 0, SEEK_SET);

        if(0 == len)
        {
            len = fileLen;
        }
        else
        {
            if(prot & PROT_WRITE)
            {
                if(fileLen < len)
                {
                    lseek(fd, len - 1, SEEK_SET);
                    write(fd, "", 1);
                }
            }
        }
    }

    pvMap = mmap(NULL, len, prot, flags, fd, offset);

    if(-1 != fd)
    {
        _close(fd);
    }

	if(NULL != plen)
	{
		*plen = len;
	}

    return pvMap;
}

/* ////////////////////////////////////////////////////////////////////////// */
